
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>

#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <getopt.h>

#include <linux/joystick.h>

#include "net_dixonpeugh_j3d_linux_JoystickInputDevice.h"

#define NAME_LENGTH 128
#define FILENAME_LENGTH 128
#define MAX_DEVICES 16

#define JS_SELECT_UDELAY 25000

struct device {
  int fd;
  char axis;
  char buttons;
  int version;
  char name[NAME_LENGTH];
  char device_filename[FILENAME_LENGTH];
} jstick[MAX_DEVICES] = {0, 2, 2, 0x000800, "Unknown"};

int devices = 0;

int newDevice( struct device *jstick, char *device_filename) {
  int errNo;

  errNo = open(device_filename, O_RDONLY);

  if (errNo < 0) return errNo;

  jstick->fd = errNo;
 
  ioctl(jstick->fd, JSIOCGVERSION, &jstick->version);
  ioctl(jstick->fd, JSIOCGAXES, &jstick->axis);
  ioctl(jstick->fd, JSIOCGBUTTONS, &jstick->buttons);
  ioctl(jstick->fd, JSIOCGNAME(NAME_LENGTH), jstick->name);
  strncpy(jstick->device_filename,device_filename,FILENAME_LENGTH);

  return errNo;
}

jint Java_net_dixonpeugh_j3d_linux_JoystickInputDevice_openDevice
   ( JNIEnv *env, jobject this, jstring deviceName )
{
    jboolean iscopy;
    jint deviceNo = devices;
 
    jclass cls = (*env)->GetObjectClass(env, this);

    jfieldID axisField, buttonsField, versionField, nameField;
    jfieldID axisDataField, buttonDataField;
    jintArray axisData, buttonData;
    jstring nameStr;

    int errNo;
    char *errText;
    jobject exception;
    
    char *device_filename;
    devices++;
    
    device_filename = (char *)(*env)->GetStringUTFChars( env, deviceName, &iscopy );
   
    errNo = newDevice( &jstick[deviceNo], device_filename );

    (*env)->ReleaseStringUTFChars(env, deviceName, device_filename);
    if (errNo > 0) 
    {
	// Get the field IDs.
	axisField       = (*env)->GetFieldID(env, cls, "numAxis", "I");
	buttonsField    = (*env)->GetFieldID(env, cls, "numButtons", "I");
	versionField    = (*env)->GetFieldID(env, cls, "version", "I");
	nameField       = (*env)->GetFieldID(env, cls,"joystickName", "Ljava/lang/String;");

	axisDataField   = (*env)->GetFieldID(env, cls, "axisData", "[I");
	buttonDataField = (*env)->GetFieldID(env, cls, "buttonData", "[I");

	// Prepare the data to be added.
	nameStr = (*env)->NewStringUTF(env, jstick[deviceNo].name);
	
	// Put the data into the Object
	(*env)->SetObjectField(env, this, nameField, nameStr);
	(*env)->SetIntField( env, this, axisField, (jint) jstick[deviceNo].axis);
	(*env)->SetIntField( env, this, buttonsField, (jint) jstick[deviceNo].buttons);
	(*env)->SetIntField( env, this, versionField, (jint) jstick[deviceNo].version);
	
      /*      
	      (*env)->SetIntArrayField( env, this, axisDataField, axisData );
	      (*env)->SetIntArrayField( env, this, buttonDataField, buttonData );
      */
	
	return deviceNo;
	
    } else {
	exception = (*env)->FindClass(env, "java/io/IOException");
      if (exception == 0) {
	return -1;
      }
      (*env)->ThrowNew(env, exception, strerror( errNo ));
      return -1;
    }
}

void Java_net_dixonpeugh_j3d_linux_JoystickInputDevice_setDeviceMode
(JNIEnv *env, jobject this, jint handle, jint mode) 
{
  if (mode) {
    // Non-Blocking Mode
    fcntl(jstick[handle].fd, F_SETFL, O_NONBLOCK);
  } else {
    // Blocking Mode -- Can't reset it.  I supose I could reopen.
  }
}
void Java_net_dixonpeugh_j3d_linux_JoystickInputDevice_closeDevice
(JNIEnv *env, jobject this, jint handle)
{
  close( jstick[handle].fd );
}

void Java_net_dixonpeugh_j3d_linux_JoystickInputDevice_pollDevice
(JNIEnv *env, jobject this, 
 jint handle, jintArray axisData, jintArray buttonData)
{
    struct js_event js;
    struct timeval tv;
    fd_set set;

    jfieldID arrayField;
    jboolean isCopy = JNI_FALSE;
    jint *intArray;
    int numRead;

    jclass cls = (*env)->GetObjectClass(env, this);

    // Lets do the select thingy, see if it works.
    tv.tv_sec = 0;
    tv.tv_usec = JS_SELECT_UDELAY;

    FD_ZERO(&set);
    FD_SET(jstick[handle].fd, &set);

    if (!select(jstick[handle].fd+1, &set, NULL, NULL, &tv))
      return;

    if (numRead = read(jstick[handle].fd, &js, sizeof(struct js_event))
	   == sizeof(struct js_event)) {
      switch (js.type & ~JS_EVENT_INIT) {
      case JS_EVENT_BUTTON:
	intArray = (*env)->GetIntArrayElements(env, buttonData, &isCopy);
	intArray[js.number] = js.value;
	(*env)->ReleaseIntArrayElements(env, buttonData, intArray, JNI_COMMIT);
	break;
      case JS_EVENT_AXIS:
	intArray = (*env)->GetIntArrayElements(env, axisData, &isCopy);
	intArray[js.number] = js.value;
	(*env)->ReleaseIntArrayElements(env, axisData, intArray, JNI_COMMIT);
	break;
      }
    }
   
    /*
    if (numRead < 0) {
	jclass exception = (*env)->FindClass(env, "java/io/IOException");
	if (exception == 0) {
	    return;
	}
	(*env)->ThrowNew(env, exception, "Unable to read device");
	return;
    }
    */
    
}



