import java.io.*;
import java.net.*;

/**
 * This class is responsable for the sending of the joint coordinates to the 
 * real robot.
 * @author Dieter Van Uytvanck
 * @version 0.1
 */
public class CyclopCommunicationSender {
    // this string holds the hostname or the ip of the 3d model
    private String hostname = "kweekvijver.elis.rug.ac.be";
    private int port = 1666;
    private Socket clientSocket;
    private DataOutputStream osClient;
    
    /** 
     * Creates a new instance of CyclopCommunicationSender and open the 
     * necessaey socket.
     */
    public CyclopCommunicationSender() {
        openSocket();
    }
    
    /** Opens the socket to send the coordinates through. */
    public void openSocket() {
        try {
            clientSocket = new Socket(hostname, port);
            osClient = new DataOutputStream(clientSocket.getOutputStream());
        }
        catch (UnknownHostException e) {
            System.err.println("Don't know about host: hostname");
        }
        catch (IOException e) {
            System.err.println("Couldn't get I/O for the connection to: hostname");
        }
    }
    
    /** Closes the opened socket. */
    public void closeSocket() {
        try {
            clientSocket.close();
        }
        catch (UnknownHostException e) {
            System.err.println("Trying to connect to unknown host: " + e);
        }
        catch (IOException e) {
            System.err.println("IOException:  " + e);
        }
    }
    
    /**
     * Sends a set of joint coordinates to the real robot.
     * @param pos A list of all joint coordinates and the gripper position.
     * The used order is: Base, Shoulder, Elbow, Wrist (tilt), Wrist (twist),
     * Gripper.
     */
    public void send(double [] pos) {
        try {
            for (int i=0; i<6; i++)
                osClient.writeFloat( (float) pos[i]);
        }
        catch (IOException e) {
            System.err.println("The following IOException occured: " + e);
        }
    }
    
}
