import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;

/**
 * A class made to hide the complexity of the creation of a ViewBranchGraph.
 * @author Dieter Van Uytvanck
 * @version 0.1
 */
public class CyclopView {
  
    private View myView;
    private TransformGroup myTransformGroup;
        
  /**
   * Creates a ViewBranchGraph.
   * @param myCanvas3D the Canvas3D where the viewbranch will reside
   */
    public CyclopView(Canvas3D myCanvas3D) {
	constructView(myCanvas3D);
	constructViewBranch();
    }

    /**
     * constructView() takes a Canvas3D reference and constructs
     * a View to display in that Canvas3D.  It uses the default
     * PhysicalBody and PhysicalEnvironment (both required to be
     * set or else the 3D runtime will throw exceptions).  The
     * returned View is used by constructViewBranch() to attach
     * the scene graph's ViewPlatform to a Canvas3D for rendering.
     *
     * @see constructViewBranch(View)
     */
    private void constructView(Canvas3D myCanvas3D) {
	myView = new View();
	myView.addCanvas3D(myCanvas3D);
	myView.setPhysicalBody(new PhysicalBody());
	myView.setPhysicalEnvironment(new PhysicalEnvironment());
    }
    
    
    /**
     * constructViewBranch() takes as input a View which we
     * attached to our Canvas3D in constructView().  It constructs
     * a default view branch for the scene graph, attaches
     * the View to the ViewPlatform, and returns a reference to
     * our Locale for use by constructContentBranch()
     * in creating content for our scene graph.
     *
     * @see constructView(Canvas3D)
     * @see constructContentBranch(Locale)
     */
    private void constructViewBranch() {
	myTransformGroup = new TransformGroup();
	myTransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	myTransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	
	ViewPlatform myViewPlatform = new ViewPlatform();
	myTransformGroup.addChild(myViewPlatform);
	myView.attachViewPlatform(myViewPlatform);
    }
    
    /**
     * Returns the TransformGroup that is the parent of the ViewPlatform.
     * @return the parent of the ViewPlatform
     */
    public TransformGroup getTransformGroup() {
	return myTransformGroup;
    }

    /**
     * Returns the View of this ViewBranchGraph.
     * @return the view
     */
    public View getView() {
	return myView;
    }

}
