import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;
import java.io.*;

/**
 * This class creates a behavior for a 3-axis, 4-button joystick in order to
 * manipulate the virtual and real robot.
 *
 * @author Dieter Van Uytvanck
 * @version 0.1
 */

public class JoystickBehavior extends javax.media.j3d.Behavior {
    
    // check every 50 ms
    private WakeupOnElapsedTime conditions = new WakeupOnElapsedTime(50);
    private TransformGroup transformGroup;
    private Sensor sensor;
    private Transform3D transform = new Transform3D();
    private int buttons[]= new int[4];
    private Cyclop refCyclop;
    
    /** 
     * Creates a new instance of JoystickBehavior.
     * @param sensor the sensor object associated with the joystick device
     * @param c the reference to the Cyclop object
     */
    public JoystickBehavior( Sensor sensor, Cyclop c) {
	this.sensor = sensor;
        this.refCyclop = c;
    }
    
    /** Starts the behavior */
    public void initialize() {
	wakeupOn(conditions);    
        System.out.println("init");
    }
    
    /** 
     * Processes the input from the joystick when a stimulus has been detected.
     * @param enumeration An enumeration of all stimuli.
     */
    public void processStimulus(java.util.Enumeration enumeration) {
        // read Axis data
	sensor.getRead(transform);
	// read Button data
	sensor.lastButtons(buttons);
     	
	// process axis data
	Vector3d vec = new Vector3d();
        transform.get(vec);
        
        if (vec.x < -0.5) 
            refCyclop.checkBaseRotate(-0.1, true);
        else if (vec.x > 0.5)
            refCyclop.checkBaseRotate(0.1, true);
        
        if (vec.y < -0.5)
            refCyclop.checkShoulderRotate(-0.1, true);
        else if (vec.y > 0.5)
            refCyclop.checkShoulderRotate(0.1, true);
        
        if (vec.z < -0.5)
            refCyclop.checkElbowRotate(-0.1, true);
        else if (vec.z > 0.5)
            refCyclop.checkElbowRotate(0.1, true);
        

	// process button data
	if(buttons[0] == 1) 
            refCyclop.checkWristTilt(0.1, true);
        else if(buttons[1] == 1)
            refCyclop.checkWristTilt(-0.1, true);
        
        if(buttons[2] == 1)
            refCyclop.checkWristTwist(0.1, true);
        else if(buttons[3] == 1)
            refCyclop.checkWristTwist(-0.1, true);

	wakeupOn(conditions);
    }

}
