import java.awt.AWTEvent;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.awt.event.*;

import javax.media.j3d.*;
import javax.vecmath.*;

import com.sun.j3d.utils.behaviors.mouse.MouseBehavior;
import com.sun.j3d.utils.behaviors.mouse.MouseBehaviorCallback;

/**
 * This class is overrides the standard MouseRotateOneAxisBehavior by moving
 * the base part of the robot by means of a method from Cyclop.
 * 
 * @author Dieter Van Uytvanck
 * @version 0.1
 * @see MouseRotateOneAxis
 * @see Cyclop
 */
public class MouseRotateOneAxisBase extends MouseRotateOneAxis {
    
    /** Creates new MouseRotateOneAxisBase */
    public MouseRotateOneAxisBase(int flags, Cyclop c) {
	super(flags, c);
    }
    
    /**
     * This method is invoked by the behavior scheduler.
     * It overrides the processStimulus method: instead of performing a rotation
     * directly, it calls the CheckBaseRotate method of Cyclop.
     * @param criteria An enumeration of the criteria that caused the stimulus.
     */
    public void processStimulus(Enumeration criteria) {
        

	WakeupCriterion wakeup;
        AWTEvent[] events;
        MouseEvent evt;
        int id;
        int dx, dy;
        
	
	
        while (criteria.hasMoreElements()) {
            wakeup = (WakeupCriterion) criteria.nextElement();
            if (wakeup instanceof WakeupOnAWTEvent) {
                events = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
                if (events.length > 0) {
                    evt = (MouseEvent) events[events.length-1];
                    processMouseEvent(evt);
                    if (((buttonPress)&&((flags & MANUAL_WAKEUP) == 0)) ||
                    ((wakeUp)&&((flags & MANUAL_WAKEUP) != 0))) {
                        id = evt.getID();
                        if ((id == MouseEvent.MOUSE_DRAGGED) &&
                        !evt.isMetaDown() && ! evt.isAltDown()){
                            x = evt.getX();
                            y = evt.getY();
                            
                            dx = x - x_last;
                            dy = y - y_last;
                            
                            if (!reset){
                                x_angle = dy * y_factor;
                                y_angle = dx * x_factor;
                                
                                if (axisOfRotation == X_AXIS) {
				     refCyclop.checkBaseRotate(y_angle, true);
                                }
                                // else: do nothing!
                                
				if (callback!=null)
                                    callback.transformChanged(MouseBehaviorCallback.ROTATE,
                                    currXform );
                            }
                            else {
                                reset = false;
                            }
                            
                            x_last = x;
                            y_last = y;
                        }
                        else if (id == MouseEvent.MOUSE_PRESSED) {
                            x_last = evt.getX();
                            y_last = evt.getY();
                        }
                    }
                }
            }
        }
        
	wakeupOn(mouseCriterion);
	
    }
    
}
